@@ -11,6 +11,7 @@ import unittest
 from six.moves import mock
 
 from code_include import error_classes
+from code_include import extension
 
 from .. import common
 
@@ -47,7 +48,9 @@ class Inputs(unittest.TestCase):
                 this test will forcibly raise the found exception.
 
         """
-        data = common.load_cache(os.path.join(_CURRENT_DIRECTORY, "fake_project", "objects.inv"))
+        data = common.load_cache(
+            os.path.join(_CURRENT_DIRECTORY, "fake_project", "objects.inv")
+        )
 
         _get_app_inventory.return_value = data
         _reraise_exception.return_value = True
@@ -67,25 +70,207 @@ class Inputs(unittest.TestCase):
     def test_incorrect_directive_target(self):
         """Check that a bad tag like ":foo:" raises the expected exception."""
         self._test(  # pylint: disable=no-value-for-parameter
-            [u":nonexistent:tag:`some.module.that.may.exist`"],
-            error_classes.MissingDirective,
+            [":nonexistent:tag:`some.module.that.may.exist`"],
+            error_classes.MissingTag,
         )
 
     def test_incorrect_namespace(self):
-        """Check that a valid tag but incorrect namespace raises the expected exception."""
+        """Fail to generate text because the tag's namespace is missing."""
         self._test(  # pylint: disable=no-value-for-parameter
-            [u":meth:`path.that.does.not.exist`"], error_classes.MissingNamespace
+            [":meth:`path.that.does.not.exist`"], error_classes.MissingNamespace
         )
 
 
-class _Common(unittest.TestCase):
+class ContentsStore(unittest.TestCase):
+    """A class that provides input text for other unittest classes.
+
+    The text in this class mimics what the user would write in a regular
+    code-include block.
+
+    """
+
+    @staticmethod
+    def _get_fake_project_class():
+        return [":class:`fake_project.basic.MyKlass`"]
+
+    @staticmethod
+    def _get_fake_project_function():
+        return [":func:`fake_project.basic.set_function_thing`"]
+
+    @staticmethod
+    def _get_fake_project_module():
+        return [":mod:`fake_project.basic`"]
+
+    @staticmethod
+    def _get_fake_project_nested_class():
+        return [":class:`fake_project.nested_folder.another.MyKlass`"]
+
+    @staticmethod
+    def _get_fake_project_nested_function():
+        return [":func:`fake_project.nested_folder.another.set_function_thing`"]
+
+    @staticmethod
+    def _get_fake_project_nested_private_function():
+        return [
+            ":func:`fake_project.nested_folder.another._set_private_function_thing`"
+        ]
+
+    @staticmethod
+    def _get_fake_project_nested_method():
+        return [":meth:`fake_project.nested_folder.another.MyKlass.get_method`"]
+
+    @staticmethod
+    def _get_fake_project_nested_module():
+        return [":mod:`fake_project.nested_folder.another`"]
+
+    @staticmethod
+    def _get_fake_project_private_function():
+        return [":func:`fake_project.basic._set_private_function_thing`"]
+
+    @staticmethod
+    def _get_fake_project_method():
+        return [":meth:`fake_project.basic.MyKlass.get_method`"]
+
+
+class Linking(ContentsStore):
+    """A class that checks if linking to source code and documentation works."""
+
+    @staticmethod
+    @mock.patch("code_include.source_code._get_source_module_data")
+    @mock.patch("code_include.source_code._get_source_code_from_object")
+    @mock.patch("code_include.source_code._get_app_inventory")
+    def _get_nodes(
+        data, content, _inventory, _get_from_object, _get_source_module_data
+    ):
+        cache = common.load_cache(
+            os.path.join(_CURRENT_DIRECTORY, "fake_project", "objects.inv")
+        )
+
+        _inventory.return_value = cache
+        _get_source_module_data.return_value = data
+        _get_from_object.return_value = ""
+
+        directive = common.make_mock_directive(content)
+
+        return directive.run()
+
+    @mock.patch("code_include.source_code._get_source_code_from_object")
+    @mock.patch("code_include.extension.Directive._is_link_requested")
+    @mock.patch("code_include.extension.Directive._needs_unindent")
+    def test_link_to_documentation(
+        self,
+        _needs_unindent,
+        _is_link_requested,
+        _get_source_code_from_object,
+    ):
+        """Link to the original page where Python source-code was found.
+
+        Args:
+            _needs_unindent (:class:`mock.mock.MagicMock`):
+                The patched function that controls indentation of code-include.
+            _is_link_requested (:class:`mock.mock.MagicMock`):
+                This adds a hyperlink to the original Python documentation.
+            _get_source_code_from_object (:class:`mock.mock.MagicMock`):
+                Disable reading from source-code. This forces
+                sphinx-code-include to read from a Sphinx inventory
+                file.
+
+        """
+        _needs_unindent.return_value = False
+        _is_link_requested.return_value = True
+        _get_source_code_from_object.return_value = ""
+
+        data = (
+            os.path.join(
+                _CURRENT_DIRECTORY,
+                "fake_project",
+                "_modules",
+                "fake_project",
+                "basic.html",
+            ),
+            "MyKlass.get_method",
+        )
+        content = self._get_fake_project_method()
+        nodes = self._get_nodes(data, content)  # pylint: disable=no-value-for-parameter
+
+        self.assertEqual(2, len(nodes))
+        self.assertTrue(
+            any(
+                node
+                for node in nodes
+                if isinstance(
+                    node,
+                    extension._DocumentationHyperlink,  # pylint: disable=protected-access
+                )
+            )
+        )
+
+    @mock.patch("code_include.source_code._get_source_code_from_object")
+    @mock.patch("code_include.extension.Directive._is_source_requested")
+    @mock.patch("code_include.extension.Directive._needs_unindent")
+    def test_link_to_source(
+        self,
+        _needs_unindent,
+        _is_source_requested,
+        _get_source_code_from_object,
+    ):
+        """Link to the original page where Python source-code was found.
+
+        Args:
+            _needs_unindent (:class:`mock.mock.MagicMock`):
+                The patched function that controls indentation of code-include.
+            _is_source_requested (:class:`mock.mock.MagicMock`):
+                This adds a hyperlink to the original Python source-code.
+            _get_source_code_from_object (:class:`mock.mock.MagicMock`):
+                Disable reading from source-code. This forces
+                sphinx-code-include to read from a Sphinx inventory
+                file.
+
+        """
+        _needs_unindent.return_value = False
+        _is_source_requested.return_value = True
+        _get_source_code_from_object.return_value = ""
+
+        data = (
+            os.path.join(
+                _CURRENT_DIRECTORY,
+                "fake_project",
+                "_modules",
+                "fake_project",
+                "basic.html",
+            ),
+            "MyKlass.get_method",
+        )
+        content = self._get_fake_project_method()
+        nodes = self._get_nodes(data, content)  # pylint: disable=no-value-for-parameter
+
+        self.assertEqual(2, len(nodes))
+        self.assertTrue(
+            any(
+                node
+                for node in nodes
+                if isinstance(
+                    node,
+                    extension._SourceCodeHyperlink,  # pylint: disable=protected-access
+                )
+            )
+        )
+
+
+class _Common(ContentsStore):
     """A base class which is used by sub-classes to make tests more concise."""
 
     @mock.patch("code_include.source_code._get_source_module_data")
     @mock.patch("code_include.source_code._get_source_code_from_object")
     @mock.patch("code_include.source_code._get_app_inventory")
     def _test(
-        self, data, content, expected, _inventory, _get_from_object, _get_source_module_data
+        self,
+        data,
+        content,
+        expected,
+        _inventory,
+        _get_from_object,
+        _get_source_module_data,
     ):
         """A generic function that tests a code-include directive for some text.
 
@@ -108,7 +293,9 @@ class _Common(unittest.TestCase):
                 point of this function - testing generated source-code.
 
         """
-        cache = common.load_cache(os.path.join(_CURRENT_DIRECTORY, "fake_project", "objects.inv"))
+        cache = common.load_cache(
+            os.path.join(_CURRENT_DIRECTORY, "fake_project", "objects.inv")
+        )
 
         _inventory.return_value = cache
         _get_source_module_data.return_value = data
@@ -137,7 +324,7 @@ class RenderText(_Common):
             ),
             "MyKlass.get_method",
         )
-        content = [u":meth:`fake_project.basic.MyKlass.get_method`"]
+        content = self._get_fake_project_method()
 
         expected = textwrap.dedent(
             '''\
@@ -160,7 +347,7 @@ class RenderText(_Common):
             ),
             "MyKlass",
         )
-        content = [u":class:`fake_project.basic.MyKlass`"]
+        content = self._get_fake_project_class()
 
         expected = textwrap.dedent(
             '''\
@@ -211,7 +398,7 @@ class RenderText(_Common):
             ),
             "set_function_thing",
         )
-        content = [u":func:`fake_project.basic.set_function_thing`"]
+        content = self._get_fake_project_function()
 
         expected = textwrap.dedent(
             '''\
@@ -241,7 +428,7 @@ class RenderText(_Common):
             ),
             "_set_private_function_thing",
         )
-        content = [u":func:`fake_project.basic._set_private_function_thing`"]
+        content = self._get_fake_project_private_function()
 
         expected = textwrap.dedent(
             '''\
@@ -271,7 +458,7 @@ class RenderText(_Common):
             ),
             "",
         )
-        content = [u":mod:`fake_project.basic`"]
+        content = self._get_fake_project_module()
 
         expected = textwrap.dedent(
             '''\
@@ -402,9 +589,7 @@ class RenderText(_Common):
 
 
 class RenderTextNested(_Common):
-    """A class that checks to make sure projects get and return the right code.
-
-    """
+    """A class that checks to make sure projects get and return the right code."""
 
     def test_get_from_html(self):
         """Check that a basic HTML file can be read."""
@@ -419,7 +604,7 @@ class RenderTextNested(_Common):
             ),
             "MyKlass.get_method",
         )
-        content = [u":meth:`fake_project.nested_folder.another.MyKlass.get_method`"]
+        content = self._get_fake_project_nested_method()
 
         expected = textwrap.dedent(
             '''\
@@ -443,7 +628,8 @@ class RenderTextNested(_Common):
             ),
             "MyKlass",
         )
-        content = [u":class:`fake_project.nested_folder.another.MyKlass`"]
+
+        content = self._get_fake_project_nested_class()
 
         expected = textwrap.dedent(
             '''\
@@ -495,7 +681,7 @@ class RenderTextNested(_Common):
             ),
             "set_function_thing",
         )
-        content = [u":func:`fake_project.nested_folder.another.set_function_thing`"]
+        content = self._get_fake_project_nested_function()
 
         expected = textwrap.dedent(
             '''\
@@ -526,7 +712,7 @@ class RenderTextNested(_Common):
             ),
             "_set_private_function_thing",
         )
-        content = [u":func:`fake_project.nested_folder.another._set_private_function_thing`"]
+        content = self._get_fake_project_nested_private_function()
 
         expected = textwrap.dedent(
             '''\
@@ -557,7 +743,8 @@ class RenderTextNested(_Common):
             ),
             "",
         )
-        content = [u":mod:`fake_project.nested_folder.another`"]
+
+        content = self._get_fake_project_nested_module()
 
         expected = textwrap.dedent(
             '''\
@@ -688,7 +875,7 @@ class RenderTextNested(_Common):
 
 
 class Options(_Common):
-    """A generic class that tests options that users can add to a code-include directive."""
+    """Make sure code-include directive options work for :obj: tags."""
 
     @mock.patch("code_include.source_code._get_source_code_from_object")
     @mock.patch("code_include.extension.Directive._needs_unindent")
@@ -713,7 +900,8 @@ class Options(_Common):
             ),
             "MyKlass.get_method",
         )
-        content = [u":meth:`fake_project.basic.MyKlass.get_method`"]
+        content = self._get_fake_project_method()
+
         expected = '''\
     def get_method(self):
         """int: Get some value."""
@@ -725,6 +913,7 @@ class Options(_Common):
     @mock.patch("code_include.source_code._get_page_preprocessor")
     def test_preprocessor(self, _get_page_preprocessor, _get_source_code_from_object):
         """Check that the optional user-configuration function works correctly."""
+
         def _remove_comments_and_docstrings(node):
             for tag in node.find_all("span", {"class": ["c1", "sd"]}):
                 tag.decompose()
@@ -742,9 +931,9 @@ class Options(_Common):
             ),
             "set_function_thing",
         )
-        content = [u":func:`fake_project.basic.set_function_thing`"]
+        content = self._get_fake_project_function()
 
-        expected = '''\
+        expected = """\
 def set_function_thing(value, another):
     
     
@@ -754,6 +943,6 @@ def set_function_thing(value, another):
         return 2
 
     
-    return 1'''
+    return 1"""
 
         self._test(data, content, expected)  # pylint: disable=no-value-for-parameter
